/*
 asm6502.h

  Revised: 2013.05.27

 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __ASM6502_H
 #include "disasm6502.h"

/* ---- constants */

/* opt (option) bits */
#define _Prescan	0x80	/* allow temporary auto-creation */
#define _Optimize	0x40	/* make smaller codes - a file can turn this on */
#define _MirrorMask	0x3	/* all bits on = default*/
#define _SubMapMask	0x3C	/* zero = none */
#define _SubMapShift	2

/* flg (flag) bits */
#define _orgInited	0x80	/* if not set, ORG is set to ->pos on write */
#define _IntoLine	0x40	/* otherwise start of line */
#define _IntoProc	0x20	/* 'proc' to 'endproc' */
#define _PreEstID	0x10	/* pre-established ID on line (don't optimize this) */
#define _EndKeyw	0x8	/* 'end' statement reached (not EOF) */
#define _PWrError	0x4	/* writef should set this on error */
#define _LScanErr	0x2

#define _LineFlags	(_IntoLine|_PreEstID)

enum
{ /* type values - low 6 bits; bit #7 = pre-established */
	_Unestablished,
	_Default,
	_Label,
	_Equate
};

enum
{ /* section values */
	_codeSection,		/* 0x8000-0xFFFF */
	_bssSection,		/* WRAM (0x200-0x07FF) */
	_zpSection		/* Zero Page (0x00-0xFF) */
};

enum
{ /* assembler errors (other than the interpreter.h standard) */
	_e_include		=-1000,	/* not handled at low-level */
	_e_include_depth	=-1001,	/* too many include-within-include or recursion */
	_e_include_open		=-1002, /* failed to open */
	_e_off_or_on		=-1003, /* bad parameter for opt */
	_e_write_in_addr_sect	=-1004,	/* attempt to write code/data in ZP/BSS */
	_e_prg_write_error	=-1005, /* writef set _PWrError */
	_e_bank_overflow	=-1006,	/* auto-bank (org $C000) wrapped */
	_e_labelscan_overflow	=-1007,	/* _LScanErr set */
	_e_code_wrap		=-1008,	/* PRG address wrapped in operation */
	_e_already_in_proc	=-1009,	/* no .endproc before another .proc */
	_e_open_proc		=-1010,	/* reached end, no endproc */
	_e_no_proc		=-1011,	/* hit endproc, not in proc */
	_e_name_too_long	=-1012	/* total name too long */
};

enum
{ /* corresponding index is +OPN_COUNT */
	_incbin		= OPN_COUNT,
	_include,
	_addr,
	_align,
	_bank,
	_block,
	_bss,
	_byt,
	_code,
	_db,
	_dcb,
	_dc_b,
	_default,
	_defchr,
	_dw,
	_dcw,
	_dc_w,
	_end,
	_endproc,
	_equ,
	_fill,
	_index,
	_ineschr,
	_inesmap,
	_inesmir,
	_inesprg,
	_mem,
	_module,
	_opt,
	_org,
	_p02,
	_pad,
	_proc,
	_processor,
	_res,
	_segment,
	_subroutine,
	_zeropage,
	_zp,
	_off,
	_on
};

/* ---- typedefs */

typedef struct asm6502frame
{
	uint8		opt, flg;
	uint8		bsz;	/* bank size, in bits */
	uint8		sct;	/* section */
	uint16		org, pos; /* pos is the actual writing position */
	int16		bnk, maxprg, maxchr; /* 0xFFF maximum */
	int16		map;	/* mapper type/index (or negative) */

	/* null term'd list: [hash.8] [pstr] [len.8] [typ.8] [val.32] */
	uint8		*lab;
	/* current module name (or NULL) */
	char		*mod;

	/* when writing bytes; can be NULL (i.e., check syntax) */
	/* IMPORTANT: this function is expected to use and update `pos`, not `org` */
	void (*writef)(struct asm6502frame*,
		uint8);
	void		*u;	/* user-defined */
} asm6502frame;

/* ---- prototypes */

/* returns pstr offset (positive), ~end, zero on error */
int32 X6502_AsmGetID(const char *s,
	const char *e,
	uint8 *n, /* buffer capacity >=256 */
	const asm6502frame *f,
	int *z); /* full ID length stored in `n` */
/* returns data offset; negated if already added, zero on error */
int X6502_AsmAddID(const char *s,
	const char *e,
	uint8 *n, /* buffer capacity >=256 */
	asm6502frame *f,
	uint8 t); /* see type values above */

/* returns the total parsed length, and larger operand mode in `m` */
long X6502_ParseOperands(const char *str,
	const char *e,
	evalvalue *v,
	asm6502frame *f,
	int *m);
/* returns index */
int X6502_ParseKey(const char *s,
	const char *e,
	uint32 max);
/* returns text length parsed, error if negative */
long X6502_ParseName(const char *str,
	const char *e,
	long *z, /* out actual length */
	char **b,
	const char *parent);

/* returns error on negative, warning on positive */
long X6502_Assemble(const char *s,
	const char *e,
	int *err, /* can be NULL */
	asm6502frame*, /* zero this out first thing; can be NULL (one call) */
	const char *fname); /* can be NULL */

int X6502_AsmFile(struct FILE*,
	const char *fname, /* can be NULL */
	asm6502frame*,
	uint8 includeDepth);

int X6502_FreeFrame(asm6502frame*);
#endif
